import os
import re
import time
import sys
from yum.plugins import PluginYumExit, TYPE_CORE, TYPE_INTERACTIVE

requires_api_version = '2.6'
plugin_type = (TYPE_CORE, )
repover = []
product_releasef = "/etc/openvz-release"
product_name = ""
product_distro = "cpe:/o:openvzproject:vz:7"
product_postinstall_script = "/usr/libexec/update-post"
installroot = ""

def calculate_ver(m):
    version = 0
    if m and m.group(1) and m.group(2) and m.group(3) and m.group(4):
        version = int(m.group(1)) * pow(100,7) + int(m.group(2)) * pow(100,6) \
            + int(m.group(3)) * pow(100,4) + int(m.group(4))
    return version

def getver_str(ver):
    major = ver / pow(100,7)
    middle = (ver - major * pow(100,7)) / pow(100,6)
    minor = (ver - major * pow(100,7) - middle * pow(100,6)) / pow(100,4)
    release = ver - major * pow(100,7) - middle * pow(100,6) - minor * pow(100,4)
    return "%i.%i.%i (%i)" % (major, middle, minor, release)

def postreposetup_hook(conduit):
    for repo in conduit.getRepos().listEnabled():
        if not repo._getRepoXML().tags['distro']:
            continue
        try:
            ver = repo._getRepoXML().tags['distro'][product_distro].pop()
            vernum = calculate_ver(re.match("^.*(\d+)\.(\d+)\.(\d+) \((\d+)\)$", ver, re.I))
            repover.append(vernum)
        except:
            pass

def posttrans_hook(conduit):
    installroot = ""
    if conduit._base.conf.installroot != "/":
        installroot = conduit._base.conf.installroot
    if len(repover) == 0:
        return
    try:
        with open(installroot + product_releasef, "r+") as f:
            product_name = re.sub(" [0-9]*\.[0-9]*\.[0-9]* \([a-zA-Z0-9\.]*\).*", "", f.read()).rstrip()
            f.seek(0)
            f.write("%s %s\n" % (product_name, getver_str(sorted(repover, reverse=True)[0])))
            f.truncate()
    except:
        conduit.error(2, "Warning: Failed to modify %s" % product_releasef)

def close_hook(conduit):
    # exec here post-hook script from bm-release if any
    if not os.path.exists(installroot + product_postinstall_script):
        return

    err = "Warning: Failed to exec product post-install %s script" % product_postinstall_script

    # Do the fork due of anaconda issues with root
    try:
        pid = os.fork()
        if pid:
            rc = os.waitpid(pid, 0)[1]
        else:
            if installroot:
                os.chroot(installroot)
            os.execl(product_postinstall_script, product_postinstall_script)
    except:
        conduit.error(2, err)

    if rc == 0:
        return
    conduit.error(2, err)
